MIME_TYPES = {
  "aac"     :	"audio/aac",
  "abw"     :	"application/x-abiword",
  "arc"     :	"application/x-freearc",
  "avi"     :	"video/x-msvideo",
  "azw"     :	"application/vndamazonebook",
  "bin"     :	"application/octet-stream",
  "bmp"     :	"image/bmp",
  "bz"      :	"application/x-bzip",
  "bz2"     :	"application/x-bzip2",
  "csh"     :	"application/x-csh",
  "css"     :	"text/css",
  "csv"     :	"text/csv",
  "doc"     :	"application/msword",
  "docx"    :	"application/vndopenxmlformats-officedocumentwordprocessingmldocument",
  "eot"     :	"application/vndms-fontobject",
  "epub"    :	"application/epub+zip",
  "gz"      :	"application/gzip",
  "gif"     :	"image/gif",
  "htm"     :	"text/html",
  "html"    :	"text/html",
  "ico"     :	"image/vndmicrosofticon",
  "ics"     :	"text/calendar",
  "jar"     :	"application/java-archive",
  "jpeg"    :	"image/jpeg",
  "js"      :	"text/javascript",
  "json"    :	"application/json",
  "jsonld"  :	"application/ld+json",
  "mid"     :	"audio/midi audio/x-midi",
  "mjs"     :	"text/javascript",
  "mp3"     :	"audio/mpeg",
  "mpeg"    :	"video/mpeg",
  "mpkg"    :	"application/vndappleinstaller+xml",
  "odp"		  :	"application/vndoasisopendocumentpresentation",
  "ods"		  :	"application/vndoasisopendocumentspreadsheet",
  "odt"		  :	"application/vndoasisopendocumenttext",
  "oga"		  :	"audio/ogg",
  "ogv"		  :	"video/ogg",
  "ogx"		  :	"application/ogg",
  "opus"		:	"audio/opus",
  "otf"		  :	"font/otf",
  "png"		  :	"image/png",
  "pdf"		  :	"application/pdf",
  "php"		  :	"application/x-httpd-php",
  "ppt"		  :	"application/vndms-powerpoint",
  "pptx"		:	"application/vndopenxmlformats-officedocumentpresentationmlpresentation",
  "rar"		  :	"application/vndrar",
  "rtf"		  :	"application/rtf",
  "sh"		  :	"application/x-sh",
  "svg"		  :	"image/svg+xml",
  "swf"		  :	"application/x-shockwave-flash",
  "tar"		  :	"application/x-tar",
  "tif"		  :	"image/tiff",
  "ts"		  :	"video/mp2t",
  "ttf"		  :	"font/ttf",
  "txt"		  :	"text/plain",
  "vsd"		  :	"application/vndvisio",
  "wav"		  :	"audio/wav",
  "weba"		:	"audio/webm",
  "webm"		:	"video/webm",
  "webp"		:	"image/webp",
  "woff"		:	"font/woff",
  "woff2"	  :	"font/woff2",
  "xhtml"	  :	"application/xhtml+xml",
  "xls"		  :	"application/vndms-excel",
  "xlsx"		:	"application/vndopenxmlformats-officedocumentspreadsheetmlsheet",
  "xml"		  :	"text/xml",
  "xul"		  :	"application/vndmozillaxul+xml",
  "zip"		  :	"application/zip",
  "3gp"		  :	"audio/3gpp",
  "3g2"     :	"audio/3gpp2",
  "7z"      :	"application/x-7z-compressed",
  "folder"  : "application/vnd.google-apps.folder",
  "dir"     : "application/vnd.google-apps.folder",
  "unknown" : "application/unknown"
}