import os,sys


def gen_css(config):
  pass

def gen_table_head(config):
  
  return ''

def gen_table_row(config):
  
  return ''
  
def gen_table_cell(config):
  
  return ''  

def gen_table_tail(config):
  
  return ''    

def gen_html_head(config):
  
  return ''      
  
def gen_html_tail(config):
  
  return ''        

def gen_button(config):
  
  return ''        